<?php

namespace Shopware\trus1TrustedShopsBadge\Components;

use Shopware\Components\Model\ModelManager;
use Shopware\Models\Shop\Repository;

/**
 * Class Configuration
 * @package Shopware\trus1TrustedShopsBadge\Components
 * @author    Trusted Shops GmbH
 * @copyright 2017 Trusted Shops GmbH
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @link      http://www.trustedshops.de/
 */
class Configuration
{
    /**
     * @var ModelManager
     */
    private $modelManager;

    /**
     * @var \Zend_Locale
     */
    private $locale;

    /**
     * list of supported locales
     *
     * @var array
     */
    protected $supportedLocales = array(
        'de_DE',
        'en_GB',
        'fr_FR',
        'es_ES',
        'it_IT',
        'nl_NL',
        'pl_PL',
    );

    /**
     * locale to use if no appropriate one was found
     *
     * @var string
     */
    protected $fallbackLocale = 'en_GB';

    /**
     * Configuration constructor.
     * @param ModelManager $modelManager
     * @param \Zend_Locale $locale
     */
    public function __construct(ModelManager $modelManager, \Zend_Locale $locale)
    {
        $this->modelManager = $modelManager;
        $this->locale = $locale;
    }

    public function getLocale()
    {

        $shopLocale = $this->locale;
        if (in_array($shopLocale, $this->supportedLocales)) {
            return $shopLocale;
        }

        // find base locale
        $localeParts = explode('_', $shopLocale);
        foreach ($this->supportedLocales as $supportedLocale) {
            if (strpos($supportedLocale, $localeParts[0]) !== false) {
                return $supportedLocale;
            }
        }

        return $this->fallbackLocale;
    }

    /**
     * @param $filenameCache
     * @param int $timeout
     * @return bool
     */
    public function trustedShopsCacheCheck($filenameCache, $timeout = 10800)
    {
        if (file_exists($filenameCache) && time() - filemtime($filenameCache) < $timeout) {
            return true;
        }
        return false;
    }

    public function getRichSnippetsData($cacheFileName, $cacheTimeOut, $apiUrl)
    {
        $reviewsFound = false;
        if (!$this->trustedShopsCacheCheck($cacheFileName, $cacheTimeOut)) {
            // load fresh from API
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_HEADER, false);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POST, false);
            curl_setopt($ch, CURLOPT_URL, $apiUrl);
            $output = curl_exec($ch);
            curl_close($ch);
            // Write the contents back to the file
            // Make sure you can write to file's destination
            file_put_contents($cacheFileName, $output);
        }

        if ($jsonObject = json_decode(file_get_contents($cacheFileName), true)) {
            $result = $jsonObject['response']['data']['shop']['qualityIndicators']['reviewIndicator']['overallMark'];
            $count = $jsonObject['response']['data'] ['shop']['qualityIndicators']['reviewIndicator']['activeReviewCount'];
            $shopName = $jsonObject['response']['data']['shop']['name'];
            $max = "5.00";
            if ($count > 0) {
                $reviewsFound = true;
            }
        }

        return array(
            'reviewsFound' => $reviewsFound,
            'shopName' => $shopName,
            'result' => $result,
            'max' => $max,
            'count' => $count);
    }

    /**
     * @param $shopId
     * @return array
     */
    public function getTrustedShopBasicConfig($shopId)
    {
        $tsConf = $this->getSettings($shopId);
        $cacheFileName = Shopware()->Container()->get('kernel')->getRootDir() . DIRECTORY_SEPARATOR . $tsConf['trustedShopsId'] . '.json';
        $cacheTimeOut = 43200; // half a day
        $apiUrl = 'http://api.trustedshops.com/rest/public/v2/shops/' . $tsConf['trustedShopsId'] . '/quality/reviews.json';


        $trustedShop = array(
            'locale' => $this->getLocale(),
            'id' => $tsConf['trustedShopsId'],
            'expertMode' => $tsConf['expertMode'],
            'badgeVariant' => $tsConf['badgeVariant'],
            'trustedShopsTrustBadgeCode' => $tsConf['trustedShopsTrustBadgeCode'],
            'trustedShopsShowReviewStickerActive' => $tsConf['trustedShopsShowReviewStickerActive'],
            'trustedShopsShowReviewStickerCode' => $tsConf['trustedShopsShowReviewStickerCode'],
            'trustedShopsShowRatingsStarsActive' => $tsConf['trustedShopsShowRatingsStarsActive'],
            'trustedShopsShowRatingsStarsCode' => $tsConf['trustedShopsShowRatingsStarsCode'],
            'prodReviewStickerTabName' => $tsConf['prodReviewStickerTabName'],
            'yOffset' => $tsConf['yOffset'],
            'prodStickerBorderColor' => $tsConf['prodStickerBorderColor'],
            'prodStickerStarColor' => $tsConf['prodStickerStarColor'],
            'starColor' => $tsConf['starColor'],
            'starSize' => $tsConf['starSize'],
            'fontSize' => $tsConf['fontSize'],
            'reviewStickerBackgroundColor' => $tsConf['reviewStickerBackgroundColor'],
            'reviewStickerStarSize' => $tsConf['reviewStickerStarSize'],
            'trustedShopsInStockDeliveryTime' => $tsConf['trustedShopsInStockDeliveryTime'],
            'trustedShopsNotInStockDeliveryTime' => $tsConf['trustedShopsNotInStockDeliveryTime'],
            'hideEmptyStarRatings' => $tsConf['hideEmptyStarRatings'],
            'collectRatings' => $tsConf['collectRatings'],
            'collectRatingsExpert' => $tsConf['collectRatingsExpert'],
            'trustedShopsShowReviewStickerActiveExpert' => $tsConf['trustedShopsShowReviewStickerActiveExpert'],
            'trustedShopsShowRatingsStarsActiveExpert' => $tsConf['trustedShopsShowRatingsStarsActiveExpert'],
            'customerRatingsActive' => $tsConf['customerRatingsActive'],
            'customerRatingsActiveExpert' => $tsConf['customerRatingsActiveExpert'],
            'customerRatingsExpertCode' => $tsConf['customerRatingsExpertCode'],
            'customerRatingsFontFamily' => $tsConf['customerRatingsFontFamily'],
            'customerRatingsReviews' => $tsConf['customerRatingsReviews'],
            'customerRatingsBetterThan' => $tsConf['customerRatingsBetterThan'],
            'customerRatingsBackgroundColor' => $tsConf['customerRatingsBackgroundColor'],
            'richSnippetsActive' => $tsConf['richSnippetsActive'],
            'richSnippetsLocations' => explode(',', $tsConf['richSnippetsLocations']),
            'reviewStickerSelector' => $tsConf['reviewStickerSelector'],
            'reviewStarsSelector' => $tsConf['reviewStarsSelector'],
            'customerReviewsSelector' => $tsConf['customerReviewsSelector'],
            'richSnippetsConfig' => array(
                $this->getRichSnippetsData($cacheFileName, $cacheTimeOut, $apiUrl)
            ),
            'richSnippetsLocationsExpert' => explode(',', $tsConf['richSnippetsLocationsExpert']),
            'richSnippetsActiveExpert' => $tsConf['richSnippetsActiveExpert'],
            'richSnippetsExpertCode' => $tsConf['richSnippetsExpertCode'],
        );
        return $trustedShop;
    }

    /**
     * @param $shopId
     * @return array
     */
    public function getSettings($shopId)
    {
        // fallback if no certain shop is selected (use default shop)
        if (!$shopId) {
            /** @var Repository $shopRepository */
            $shopRepository = Shopware()->Models()->getRepository('Shopware\Models\Shop\Shop');
            $shop = $shopRepository->getActiveDefault();
            $shopId = $shop->getId();
        }

        // request database for shop specific module configurations
        $builder = Shopware()->Models()->createQueryBuilder();
        $builder->select(array('shopId'))
            ->from('Shopware\CustomModels\TrustedShopsBadge\TrustedShopsBadge', 'shopId')
            ->where('shopId = :shopId')
            ->setParameters(array('shopId' => $shopId));
        $data = $builder->getQuery()->getArrayResult();

        // if case of missing data (e.g. first use of module) load module defaults
        $defaults = $this->getDefaultData($shopId, $builder);
        if (array_key_exists(0, $data)) {
            $data = $data[0];
        } else {
            $data = $defaults;
        }

        array_walk($data, function (&$val, $key) use ($defaults) {
            if(is_null($val)) {
                $val = $defaults[$key];
            }
        });

        $cacheFileName = Shopware()->Container()->get('kernel')->getRootDir() . DIRECTORY_SEPARATOR . $data['trustedShopsId'] . '.json';
        $cacheTimeOut = 43200; // half a day
        $apiUrl = 'http://api.trustedshops.com/rest/public/v2/shops/' . trim($data['trustedShopsId']) . '/quality/reviews.json';
        $richSnippetsData = $this->getRichSnippetsData($cacheFileName, $cacheTimeOut, $apiUrl);

        if (trim($data['trustedShopsId']) != '') {
            // For the expert mode we replace the placeholder (e.g. "%tsid%")
            // within the custom javascript code. Only processed if trustedShopsId set
            $placeholdersForReplacement = array(
                '%tsid%' => $data['trustedShopsId'],

            );
            foreach ($placeholdersForReplacement as $_placeholder => $_replaceValue) {
                $data['trustedShopsTrustBadgeCode'] = str_replace($_placeholder, $_replaceValue, $data['trustedShopsTrustBadgeCode']);
                $data['trustedShopsShowReviewStickerCode'] = str_replace($_placeholder, $_replaceValue, $data['trustedShopsShowReviewStickerCode']);
                $data['trustedShopsShowRatingsStarsCode'] = str_replace($_placeholder, $_replaceValue, $data['trustedShopsShowRatingsStarsCode']);
                $data['customerRatingsExpertCode'] = str_replace($_placeholder, $_replaceValue, $data['customerRatingsExpertCode']);

            }
        }
        return $data;
    }

    /**
     * @param $shopId
     * @param $builder
     * @return mixed
     */
    protected function getDefaultData($shopId, $builder)
    {
        $shopRepository = Shopware()->Models()->getRepository('Shopware\Models\Shop\Shop');
        $shop = $shopRepository->getActiveDefault();

        $builder->setParameters(array('shopId' => $shop->getId()));
        $tsData = $builder->getQuery()->getArrayResult();

        if (array_key_exists(0, $tsData)) {
            $tsData = $tsData[0];
        }

        $tsData['id'] = $shopId;
        $tsData['shopId'] = $shopId;
        $tsData['trustedShopsId'] = '';
        $tsData['locale'] = $this->locale;
        $tsData['badgeVariant'] = 'reviews';
        $tsData['expertMode'] = false;
        $tsData['collectReviewsAndShowGuarantee'] = false;
        $tsData['trustedShopsShowReviewStickerActive'] = false;
        $tsData['prodReviewStickerTabName'] = 'Trusted Shops Bewertungen';
        $tsData['trustedShopsShowReviewStickerCode'] = '<script type="text/javascript">
      _tsProductReviewsConfig = {
            tsid: \'%tsid%\',
            sku: [\'%sku%\'],
            variant: \'productreviews\',
            borderColor: \'#0DBEDC\',
            backgroundColor: \'#ffffff\',
            element: \'#ts_product_sticker\',
            locale: \'%locale%\',
            starColor: \'#FFDC0F\',
            starSize: \'15px\',
            richSnippets: \'on\',
            ratingSummary: \'false\',
            maxHeight: \'1200px\',
            hideEmptySticker: \'false\',
            introtext: \'What our customers say about us:\'
            /* optional */
      };
      var scripts = document.getElementsByTagName(\'SCRIPT\'),
      me = scripts[scripts.length - 1];
      var _ts = document.createElement(\'SCRIPT\');
      _ts.type = \'text/javascript\';
      _ts.async = true;
      _ts.charset = \'utf-8\';
      _ts.src =\'//widgets.trustedshops.com/reviews/tsSticker/tsProductSticker.js\';
      me.parentNode.insertBefore(_ts, me);
      _tsProductReviewsConfig.script = _ts;
    </script>';

        $tsData['trustedShopsShowRatingsStarsActive'] = false;
        $tsData['trustedShopsShowRatingsStarsCode'] = '
    <script type="text/javascript" src="//widgets.trustedshops.com/reviews/tsSticker/tsProductStickerSummary.js"></script>
    <script> 
    var summaryBadge = new productStickerSummary();
    summaryBadge.showSummary(
    {
    \'tsId\': "%tsid%",
    \'sku\': [\'%sku%\'],
    \'element\': \'#ts_product_widget_position\',
    \'starColor\' : \'#FFDC0F\',
    \'starSize\' : \'14px\',
    \'fontSize\' : \'12px\',
    \'showRating\' : \'true\' ,
    \'scrollToReviews\' : \'false\' ,
    \'enablePlaceholder\': \'true\'
    }
    );
    </script> 
    ';

        $tsData['trustedShopsTrustBadgeCode'] = '<script type="text/javascript">
        (function () {
            var _tsid = \'%tsid%\';
            _tsConfig = {
              \'yOffset\': \'0\', /* offset from page bottom */
              \'variant\': \'reviews\', /* default, reviews, custom, custom_reviews */
              \'customElementId\': \'\', /* required for variants custom and custom_reviews */
              \'trustcardDirection\': \'\', /* for custom variants: topRight, topLeft, bottomRight, bottomLeft */
              \'disableResponsive\': \'false\', /* deactivate responsive behaviour */
              \'disableTrustbadge\': \'false\' /* deactivate trustbadge */
    
            };
            var _ts = document.createElement(\'script\');
            _ts.type = \'text/javascript\';
            _ts.charset = \'utf-8\';
            _ts.async = true;
            _ts.src = \'//widgets.trustedshops.com/js/\' + _tsid + \'.js\';
            var __ts = document.getElementsByTagName(\'script\')[0];
            __ts.parentNode.insertBefore(_ts, __ts);
          })();
    </script>';

        $tsData['yOffset'] = 0;
        $tsData['prodStickerBorderColor'] = '#0DBEDC';
        $tsData['prodStickerStarColor'] = '#FFDC0F';
        $tsData['starColor'] = '#FFDC0F';
        $tsData['starSize'] = 15;
        $tsData['fontSize'] = 12;
        $tsData['reviewStickerBackgroundColor'] = '#FFFFFF';
        $tsData['reviewStickerStarSize'] = 15;
        $tsData['trustedShopsInStockDeliveryTime'] = 4;
        $tsData['trustedShopsNotInStockDeliveryTime'] = 10;
        $tsData['customerRatingsActive'] = false;
        $tsData['customerRatingsActiveExpert'] = false;
        $tsData['customerRatingsExpertCode'] = ' <script type="text/javascript">
                        _tsRatingConfig = {
                            tsid: \'%tsid%\',
                            variant: \'testimonial\',
                            theme: \'light\',
                            reviews: \'5\',
                            betterThan: \'3.0\',
                            richSnippets: \'on\',
                            backgroundColor: \'#ffdc0f\',
                            linkColor: \'#000000\',
                            quotationMarkColor: \'#FFFFFF\',
                            fontFamily: \'Arial\',
                            reviewMinLength: \'10\'
                        };
                        var scripts = document.getElementsByTagName(\'SCRIPT\'),
                            me = scripts[scripts.length - 1];
                        var _ts = document.createElement(\'SCRIPT\');
                        _ts.type = \'text/javascript\';
                        _ts.async = true;
                        _ts.src =
                            \'//widgets.trustedshops.com/reviews/tsSticker/tsSticker.js\';
                        me.parentNode.insertBefore(_ts, me);
                        _tsRatingConfig.script = _ts;
                    </script>';
        $tsData['customerRatingsFontFamily'] = 'Arial';
        $tsData['customerRatingsReviews'] = 5;
        $tsData['customerRatingsBetterThan'] = '3.0';
        $tsData['customerRatingsBackgroundColor'] = '#ffdc0f';
        $tsData['richSnippetsActive'] = false;
        $tsData['richSnippetsLocations'] = '';
        $tsData['reviewStickerSelector'] = '';
        $tsData['reviewStarsSelector'] = '';
        $tsData['customerReviewsSelector'] = '';
        $tsData['richSnippetsLocationsExpert'] = '';
        $tsData['richSnippetsActiveExpert'] = false;
        $tsData['richSnippetsExpertCode'] = '<script type="application/ld+json">
                {
                "@context": "http://schema.org",
                "@type": "Organization",
                "name": "%shopname%",
                "aggregateRating" : {
                "@type": "AggregateRating",
                "ratingValue" : "%result%",
                "bestRating" : "%max%",
                "ratingCount" : "%count%"
                }
                }
            </script>';

        return $tsData;

    }
}